import re
import time


class __NowTime:

    @property
    def int(self) -> int:
        return int(self.float)

    @property
    def float(self) -> float:
        return time.time()

    @property
    def struct(self) -> tuple:
        return time.localtime()

    @property
    def str(self) -> str:
        return self.strf('%F %T')

    @staticmethod
    def strf(format: str) -> str:
        return time.strftime(format)


class Future:

    def __init__(self, future_time: ...):
        if isinstance(future_time, (int, float)):
            self.future_time = future_time
        elif isinstance(future_time, str):
            self.future_time = Time2Second(future_time)
        elif isinstance(future_time, tuple):
            self.future_time = time.mktime(future_time)
        else:
            raise TypeError(f'Unsupported parameter type: {type(future_time)}.')

    @property
    def int(self) -> int:
        return int(self.float)

    @property
    def float(self) -> float:
        return now.float + self.future_time

    @property
    def struct(self) -> tuple:
        return time.localtime(self.float)

    @property
    def str(self) -> str:
        return self.strf(format='%F %T')

    def strf(self, format: str) -> str:
        return stamp2str(self.float, format)


class Past(Future):

    @property
    def float(self) -> float:
        return now.float - self.future_time


def stamp2str(
        stamp: int or float = None,
        format: str = '%F %T',
) -> str:
    """timestamp to string time"""
    return time.strftime(format, time.localtime(stamp))


def str2stamp(
        string: str,
        format: str = '%Y-%m-%d %H:%M:%S'
) -> float:
    """string time to timestamp"""
    return time.mktime(time.strptime(string, format))


class Time2Second:
    pattern = re.compile(r'''^
        (?:(\d+(?:\.\d+)?)d)?  # day
        (?:(\d+(?:\.\d+)?)h)?  # hour
        (?:(\d+(?:\.\d+)?)m)?  # minute
        (?:(\d+(?:\.\d+)?)s)?  # second
    $''', flags=re.X)

    minute = 60
    hour = 60 * minute
    day = 24 * hour

    @classmethod
    def g(cls, x: str) -> int or float:
        if not x:
            return 0
        try:
            return int(x)
        except ValueError:
            return float(x)

    def __new__(cls, unit_time: str) -> int or float:
        d, h, m, s = cls.pattern.findall(unit_time.lower())[0]
        d, h, m, s = cls.g(d), cls.g(h), cls.g(m), cls.g(s)
        return cls.day * d + cls.hour * h + cls.minute * m + s


def second2time(sec: int or float, x: 'Not param' = '') -> str:
    """Pass in a time in seconds,
    and it will be converted to unit time."""
    if sec < 1:
        return f'{round(sec, 2)}s'

    m, s = divmod(round(sec), 60)
    h, m = divmod(m, 60)
    d, h = divmod(h, 24)

    if d:
        x += f'{d}d'
    if h:
        x += f'{h}h'
    if m:
        x += f'{m}m'
    if s:
        x += f'{s}s'

    return x


now = __NowTime()
